//
//  Simplex_NoisePlugIn.m
//  Simplex Noise
//
//  Created by Bosh on 12/23/09.
//  Copyright (c) 2009 __MyCompanyName__. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "Simplex_NoisePlugIn.h"

#define	kQCPlugIn_Name				@"Simplex Noise"
#define	kQCPlugIn_Description		@"Uses a 2D simplex noise function to generate a random value in the specified range. One of the coordinates is time, the other is the seed input. If more than one output is used, the seed is offset by \"Seed Offset\" for each consecutive port. The patch is based on code written by Stefan Gustavson \r\rhttp://staffwww.itn.liu.se/~stegu/simplexnoise/simplexnoise.pdf\r\r Bostjan Cadez (fsk) 2009"

#define F2 0.366025403784439 //0.5*(Math.sqrt(3.0)-1.0) 
#define G2 0.211324865405187 //(3.0-Math.sqrt(3.0))/6.0

@implementation Simplex_NoisePlugIn


@dynamic inputSeed,inputMinValue,inputMaxValue,inputNumberOfOctaves,inputPersistence; 
@synthesize numberOfOutputPorts,seedOffset,arrayOutput;


//copy pated from :) http://staffwww.itn.liu.se/~stegu/simplexnoise/simplexnoise.pdf

-(double) noiseWithCoordinatesX:(double)xin andY:(double)yin;
{
	
	//NSLog(@"x = %d, y = %f", xin, yin);
	double n0,n1,n2;
	double s=(xin+yin)*F2;
	int i = floor(xin+s); 
    int j = floor(yin+s); 
    double t = (i+j)*G2; 
    double X0 = i-t; // Unskew the cell origin back to (x,y) space 
    double Y0 = j-t; 
    double x0 = xin-X0; // The x,y distances from the cell origin 
    double y0 = yin-Y0; 
    // For the 2D case, the simplex shape is an equilateral triangle. 
    // Determine which simplex we are in. 
    int i1, j1; // Offsets for second (middle) corner of simplex in (i,j) coords 
    if(x0>y0) {i1=1; j1=0;} // lower triangle, XY order: (0,0)->(1,0)->(1,1) 
    else {i1=0; j1=1;}      // upper triangle, YX order: (0,0)->(0,1)->(1,1) 
    // A step of (1,0) in (i,j) means a step of (1-c,-c) in (x,y), and 
    // a step of (0,1) in (i,j) means a step of (-c,1-c) in (x,y), where 
    // c = (3-sqrt(3))/6 
    double x1 = x0 - i1 + G2; // Offsets for middle corner in (x,y) unskewed coords 
    double y1 = y0 - j1 + G2; 
    double x2 = x0 - 1.0 + 2.0 * G2; // Offsets for last corner in (x,y) unskewed coords 
    double y2 = y0 - 1.0 + 2.0 * G2;
	// Work out the hashed gradient indices of the three simplex corners 
    int ii = i & 255; 
    int jj = j & 255; 
    int gi0 = perm[ii+perm[jj]] % 12; 
    int gi1 = perm[ii+i1+perm[jj+j1]] % 12; 
    int gi2 = perm[ii+1+perm[jj+1]] % 12; 
    // Calculate the contribution from the three corners 
    double t0 = 0.5 - x0*x0-y0*y0; 
    if(t0<0) n0 = 0.0; 
    else { 
		t0 *= t0; 
		n0 = t0 * t0 * (grad3[gi0][0]*x0+grad3[gi0][1]*y0);  // (x,y) of grad3 used for 2D gradient 
    } 
    double t1 = 0.5 - x1*x1-y1*y1; 
    if(t1<0) n1 = 0.0; 
    else { 
		t1 *= t1; 
		n1 = t1 * t1 * (grad3[gi1][0]*x1+grad3[gi1][1]*y1);
		
    }
	double t2 = 0.5 - x2*x2-y2*y2; 
    if(t2<0) n2 = 0.0; 
    else { 
		t2 *= t2; 
		n2 = t2 * t2 * (grad3[gi2][0]*x2+grad3[gi2][1]*y2);
    } 
    // Add contributions from each corner to get the final noise value. 
    // The result is scaled to return values in the interval [-1,1]. 
    return 70.0 * (n0 + n1 + n2); 
} 

	

+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:
			kQCPlugIn_Name, QCPlugInAttributeNameKey, 
			kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, 
			@"Bostjan Cadez (fsk) 2009", QCPlugInAttributeCopyrightKey,
			nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	
	//Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).

	if([key isEqualToString:@"inputSeed"])
        return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Seed", QCPortAttributeNameKey,
				[NSNumber numberWithDouble:0],  QCPortAttributeDefaultValueKey,
				nil];
	if([key isEqualToString:@"inputMinValue"])
        return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Min", QCPortAttributeNameKey,
				[NSNumber numberWithDouble:-1],  QCPortAttributeDefaultValueKey,
				nil];
	if([key isEqualToString:@"inputMaxValue"])
        return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Max", QCPortAttributeNameKey,
				[NSNumber numberWithDouble:1],  QCPortAttributeDefaultValueKey,
				nil];
	if([key isEqualToString:@"inputNumberOfOctaves"])
        return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Number of Octaves", QCPortAttributeNameKey,
				[NSNumber numberWithUnsignedInt:1],  QCPortAttributeDefaultValueKey,
				[NSNumber numberWithUnsignedInt:1],  QCPortAttributeMinimumValueKey,
				nil];
	if([key isEqualToString:@"inputPersistence"])
        return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Persistence", QCPortAttributeNameKey,
				[NSNumber numberWithDouble:.5],  QCPortAttributeDefaultValueKey,
				[NSNumber numberWithDouble:0],  QCPortAttributeMinimumValueKey,
				[NSNumber numberWithDouble:1],  QCPortAttributeMaximumValueKey,
				nil];
    /*if([key isEqualToString:@"outputValue"])
        return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Value", QCPortAttributeNameKey,
				nil];*/
	
    return nil;
}

+ (NSArray*) sortedPropertyPortKeys
{
    return [NSArray arrayWithObjects:@"inputMinValue",@"inputMaxValue",@"inputSeed",@"inputNumberOfOctaves",@"inputPersistence",nil];
	
}


+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeTimeBase;
}

- (id) init
{
	if(self = [super init]) {
		/*
		Allocate any permanent resource required by the plug-in. QCPortTypeNumber
		*/
		self.numberOfOutputPorts=1;
		self.seedOffset=1;
		self.arrayOutput=NO;

		
		
		int grad2[12][3]={{1,1,0},{-1,1,0},{1,-1,0},{-1,-1,0}, 
			{1,0,1},{-1,0,1},{1,0,-1},{-1,0,-1}, 
			{0,1,1},{0,-1,1},{0,1,-1},{0,-1,-1}};
		
		for(int j=0;j<12;j++)
        for(int k=0;k<3;k++)
		grad3[j][k]=grad2[j][k];
				
		int p[] = {151,160,137,91,90,15, 
			131,13,201,95,96,53,194,233,7,225,140,36,103,30,69,142,8,99,37,240,21,10,23, 
			190, 6,148,247,120,234,75,0,26,197,62,94,252,219,203,117,35,11,32,57,177,33, 
			88,237,149,56,87,174,20,125,136,171,168, 68,175,74,165,71,134,139,48,27,166, 
			77,146,158,231,83,111,229,122,60,211,133,230,220,105,92,41,55,46,245,40,244, 
			102,143,54, 65,25,63,161, 1,216,80,73,209,76,132,187,208, 89,18,169,200,196, 
			135,130,116,188,159,86,164,100,109,198,173,186, 3,64,52,217,226,250,124,123, 
			5,202,38,147,118,126,255,82,85,212,207,206,59,227,47,16,58,17,182,189,28,42, 
			223,183,170,213,119,248,152, 2,44,154,163, 70,221,153,101,155,167, 43,172,9, 
			129,22,39,253, 19,98,108,110,79,113,224,232,178,185, 112,104,218,246,97,228, 
			251,34,242,193,238,210,144,12,191,179,162,241, 81,51,145,235,249,14,239,107, 
			49,192,214, 31,181,199,106,157,184, 84,204,176,115,121,50,45,127, 4,150,254, 
		138,236,205,93,222,114,67,29,24,72,243,141,128,195,78,66,215,61,156,180};
		for(int i=0; i<512; i++) perm[i]=p[i & 255];
		
		
	}
	
	return self;
}

- (void) setNumberOfOutputPorts: (double)input
{
	//NSLog(@"ports = %f",numberOfOutputPorts);
	double new=(input<1)?1:input;
	if(!arrayOutput)
	{
		
		double current=numberOfOutputPorts;
		int i;
		if(new>current)
		{
			for(i=(int)current;i<new;i++)
			{
				[self addOutputPortWithType:QCPortTypeNumber forKey:[NSString stringWithFormat:@"value%i",i]
							 withAttributes:
				 [NSDictionary dictionaryWithObjectsAndKeys:
				  [NSString stringWithFormat:@"Value %i",(i+1)], QCPortAttributeNameKey,
				  nil]]; 	
			}
		}
		else if(new<current)
		{
			for(i=(int)current;i>new;i--)
			{
				//NSLog(@"ports = %@",[NSString stringWithFormat:@"value%i",(i-1)]);
				[self removeOutputPortForKey:[NSString stringWithFormat:@"value%i",(i-1)]]; 	
			}
		}
	}
    numberOfOutputPorts=new;
}

-(void) setArrayOutput: (bool)input
{
	int i;
	//NSLog(@"curent = %d new = %d",arrayOutput,input);
	if(arrayOutput!=input)
	{
		if(input)
		{
			for(i=0;i<numberOfOutputPorts;i++)
			{
				//NSLog(@"ports = %@",[NSString stringWithFormat:@"value%i",(i-1)]);
				[self removeOutputPortForKey:[NSString stringWithFormat:@"value%i",(i)]]; 	
			}
			[self addOutputPortWithType:QCPortTypeStructure forKey:@"values"
						 withAttributes:
			 [NSDictionary dictionaryWithObjectsAndKeys:
			  @"Values", QCPortAttributeNameKey,
			  
			  nil]];
			[self addInputPortWithType:QCPortTypeIndex forKey:@"timeSteps"
						 withAttributes:
			 [NSDictionary dictionaryWithObjectsAndKeys:
			  @"Time Steps", QCPortAttributeNameKey,
			  [NSNumber numberWithDouble:10],  QCPortAttributeDefaultValueKey,
			  nil]];
			[self addInputPortWithType:QCPortTypeNumber forKey:@"timeStepSize"
						 withAttributes:
			 [NSDictionary dictionaryWithObjectsAndKeys:
			  @"Time Step Size", QCPortAttributeNameKey,
			  [NSNumber numberWithDouble:.1],  QCPortAttributeDefaultValueKey,
			  //[NSNumber numberWithDouble:0],  QCPortAttributeMinimumValueKey,
			  nil]];
			[self addInputPortWithType:QCPortTypeIndex forKey:@"seedSteps"
						 withAttributes:
			 [NSDictionary dictionaryWithObjectsAndKeys:
			  @"Seed Steps", QCPortAttributeNameKey,
			  [NSNumber numberWithDouble:3],  QCPortAttributeDefaultValueKey,
			  nil]];
			[self addInputPortWithType:QCPortTypeNumber forKey:@"seedStepSize"
						 withAttributes:
			 [NSDictionary dictionaryWithObjectsAndKeys:
			  @"Seed Step Size", QCPortAttributeNameKey,
			  [NSNumber numberWithDouble:1],  QCPortAttributeDefaultValueKey,
			  nil]];
			
		}
		else
		{
			//NSLog(@"we are here");
			[self removeOutputPortForKey:@"values"];
			[self removeInputPortForKey:@"timeSteps"];
			[self removeInputPortForKey:@"timeStepSize"];
			[self removeInputPortForKey:@"seedSteps"];
			[self removeInputPortForKey:@"seedStepSize"];
			for(i=(int)0;i<numberOfOutputPorts;i++)
			{
				[self addOutputPortWithType:QCPortTypeNumber forKey:[NSString stringWithFormat:@"value%i",i]
							 withAttributes:
				 [NSDictionary dictionaryWithObjectsAndKeys:
				  [NSString stringWithFormat:@"Value %i",(i+1)], QCPortAttributeNameKey,
				  nil]]; 	
			}
			
		}
	}
	arrayOutput=input;
}

- (void) finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void) dealloc
{
	/*
	Release any resources created in -init.
	*/
	
	[super dealloc];
}

+ (NSArray*) plugInKeys
{
	/*
	Return a list of the KVC keys corresponding to the internal settings of the plug-in.
	*/
	
	return [NSArray arrayWithObjects:@"numberOfOutputPorts",@"seedOffset",@"arrayOutput", nil];
}

- (id) serializedValueForKey:(NSString*)key;
{
	/*
	Provide custom serialization for the plug-in internal settings that are not values complying to the <NSCoding> protocol.
	The return object must be nil or a PList compatible i.e. NSString, NSNumber, NSDate, NSData, NSArray or NSDictionary.
	*/
	
	return [super serializedValueForKey:key];
}

- (void) setSerializedValue:(id)serializedValue forKey:(NSString*)key
{
	/*
	Provide deserialization for the plug-in internal settings that were custom serialized in -serializedValueForKey.
	Deserialize the value, then call [self setValue:value forKey:key] to set the corresponding internal setting of the plug-in instance to that deserialized value.
	*/
	
	[super setSerializedValue:serializedValue forKey:key];
}

- (QCPlugInViewController*) createViewController
{
	/*
	Return a new QCPlugInViewController to edit the internal settings of this plug-in instance.
	You can return a subclass of QCPlugInViewController if necessary.
	*/
	
	return [[QCPlugInViewController alloc] 
			initWithPlugIn:self 
			viewNibName:@"Settings"];
}

@end

@implementation Simplex_NoisePlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	*/
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	double x=time;
	double y=self.inputSeed;
	unsigned int octaves=self.inputNumberOfOctaves; 
	octaves=(octaves>1)?octaves:1;
	double persistence=self.inputPersistence;
	double minVal=self.inputMinValue;
	double maxVal=self.inputMaxValue;

	double sum;
	double frequency;
	double amplitude;
	double sumMaxAmplitude;
	NSMutableArray* array;
	
	
	if(!arrayOutput)
	{
		for(int port=0;port<numberOfOutputPorts;port++)
		{
			sum = 0.;
			frequency = 1.;
			amplitude = 1.;
			sumMaxAmplitude = 0;
			for(int i=0; i < octaves; i++ ) {
				sum += [self noiseWithCoordinatesX:x*frequency andY:(y+port*seedOffset)*frequency] * amplitude;
				frequency *= 2.;
				sumMaxAmplitude += amplitude;
				amplitude *= persistence;
			}
			
			//self.outputValue=(sum / sumMaxAmplitude*.5+.5)*(maxVal-minVal)+minVal;
			[self setValue:[NSNumber numberWithDouble:(sum / sumMaxAmplitude*.5+.5)*(maxVal-minVal)+minVal] forOutputKey:[NSString stringWithFormat:@"value%i",port]];
		}
	}
	else
	{
		unsigned int timeSteps=[[self valueForInputKey:@"timeSteps"] unsignedIntegerValue];
		unsigned int seedSteps=[[self valueForInputKey:@"seedSteps"] unsignedIntegerValue];
		double timeStepSize=[[self valueForInputKey:@"timeStepSize"] doubleValue];
		double seedStepSize=[[self valueForInputKey:@"seedStepSize"] doubleValue];
		
		if (valuesArray != nil) [valuesArray release];
		valuesArray = [[NSMutableArray alloc] initWithCapacity:timeSteps];
		
		for(int t=0;t<timeSteps;t++)
		{
			
			array=[[NSMutableArray alloc] initWithCapacity:seedSteps];
			for(int s=0;s<seedSteps;s++)
			{
				sum = 0.;
				frequency = 1.;
				amplitude = 1.;
				sumMaxAmplitude = 0;
				for(int i=0; i < octaves; i++ ) {
					sum += [self noiseWithCoordinatesX:(x+t*timeStepSize)*frequency andY:(y+s*seedStepSize)*frequency] * amplitude;
					frequency *= 2.;
					sumMaxAmplitude += amplitude;
					amplitude *= persistence;
				}
				
				[array insertObject:[NSNumber numberWithDouble:(sum / sumMaxAmplitude*.5+.5)*(maxVal-minVal)+minVal] atIndex:s];
				
			}
			[valuesArray insertObject: array atIndex: t];
			[array release];
			
		}
		NSDictionary* koko= [NSDictionary dictionaryWithObject: valuesArray forKey: @"values" ];
		[self setValue:koko forOutputKey:@"values"];
	
	}	
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
	if (valuesArray != nil) [valuesArray release];
	
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	*/
}



@end
